class ExtendedLogger < Logger
  def self.configure receiver, attribute_name=nil
    attribute_name ||= 'logger'.freeze
    setter = "#{attribute_name}=".freeze

    logger = get receiver

    receiver.public_send setter, logger
  end

  def self.get receiver
    receiver_name =
      if receiver.is_a? Module
        receiver.name
      else
        receiver.class.name
      end

    logger = Factory.()

    logger.progname = "(#{receiver_name})"
    logger
  end

  def self.instance
    @instance ||= Factory.()
  end

  def self.levels
    @levels ||= {
      -7 => 'OBSOLETE'.freeze,
      -6 => 'DATA'.freeze,
      -5 => 'TRACE'.freeze,
      -4 => 'DEBUG'.freeze,
      -3 => 'OPT_DATA'.freeze,
      -2 => 'OPT_TRACE'.freeze,
      -1 => 'OPT_DEBUG'.freeze,
      0 => 'INFO'.freeze,
      1 => 'PASS'.freeze,
      2 => 'FAIL'.freeze,
      3 => 'FOCUS'.freeze,
      4 => 'WARN'.freeze,
      5 => 'ERROR'.freeze,
      6 => 'FATAL'.freeze,
    }
  end

  def self.level? level
    levels.key? level
  end

  def format_severity severity
    unless self.class.level? severity
      Kernel.warn "(extended-logger) Unknown log severity level #{severity.inspect}"
    end

    self.class.levels.fetch severity, 'ANY'.freeze
  end

  def io
    return unless @logdev
    @logdev.dev
  end

  def unknown *arguments, &block
    add 7, *arguments, &block
  end

  levels.each do |level, name|
    define_method name.downcase do |*arguments, &block|
      add level, *arguments, &block
    end
  end
end
