class ExtendedLogger
  class Factory
    attr_reader :env

    def initialize env
      @env = env
    end

    def self.build env=nil
      env ||= ENV
      new env
    end

    def self.call *arguments
      instance = build *arguments
      instance.()
    end

    def call
      logger = ExtendedLogger.new device

      formatter = Formatter.new
      formatter.palette = Formatter.default_palette if colors? logger.io
      logger.formatter = formatter

      level = self.level
      logger.level = level if level

      logger
    end

    def colors? io
      if io.tty?
        ENV['LOG_COLOR'] != 'off'
      else
        ENV['LOG_COLOR'] == 'on'
      end
    end

    def device
      ENV['LOG_DEVICE'] || $stderr
    end

    def level
      if ENV.key? 'LOG_LEVEL'
        level , _ = ExtendedLogger.levels.rassoc ENV['LOG_LEVEL'].upcase
        level
      end
    end
  end
end
