class ExtendedLogger
  class Formatter
    attr_writer :logger_formatter
    attr_writer :palette

    def call severity, *arguments
      log_entry = delegate severity, *arguments
      color log_entry, severity
    end

    def color log_entry, severity
      colorizer = palette[severity]

      if colorizer
        log_entry = "#{colorizer}#{log_entry}\e[0m"
      end

      log_entry
    end

    def delegate *arguments, message
      message = format_message message, arguments
      logger_formatter.(*arguments, message)
    end

    def format_message message, arguments
      output = ''

      message.each_line "\n".freeze do |line|
        if line == "\n".freeze
          line = "\\n"
        else
          line.gsub! "\r".freeze, "\\r".freeze
        end

        if output.empty?
          output.concat line
        else
          formatted_line = logger_formatter.(*arguments, line.chomp("\n".freeze))
          output.concat formatted_line
        end
      end

      output.chomp "\n".freeze
    end

    def logger_formatter
      @logger_formatter ||= Logger::Formatter.new
    end

    def palette
      @palette ||= {}
    end

    def self.ansi_colors
      @ansi_colors ||= %i(black red green yellow blue magenta cyan white)
    end

    def self.col fg, brightness, bg=nil
      brightness = { :bright => 1, :normal => 0 }.fetch brightness
      escape = "\e[#{brightness};3#{ansi_colors.index fg}m"

      if bg
        escape << "\e[4#{ansi_colors.index bg}m"
      end

      escape
    end

    def self.default_palette
      @default_palette ||= {
        'OBSOLETE' => col(:black, :normal),
        'DATA' => col(:green, :normal),
        'TRACE' => col(:cyan, :normal),
        'DEBUG' => col(:blue, :normal),
        'OPT_DATA' => col(:green, :normal, :white),
        'OPT_TRACE' => col(:cyan, :normal, :white),
        'OPT_DEBUG' => col(:blue, :normal, :white),
        'PASS' => col(:white, :bright, :green),
        'FAIL' => col(:white, :bright, :red),
        'FOCUS' => col(:white, :bright, :blue),
        'WARN' => col(:yellow, :normal),
        'ERROR' => col(:red, :normal),
        'FATAL' => col(:black, :bright, :red),
        'ANY' => col(:white, :bright, :magenta),
      }
    end
  end
end
